@@ -252,14 +252,15 @@ static void *config_server_create(pool *p, server_rec *s)
     rec->szCACertificateFile  = NULL;
     rec->szLogFile            = NULL;
     rec->szReqCiphers         = NULL;
-    rec->fileLogFile          = NULL;
     rec->nVerifyDepth         = 0;
     rec->nVerifyClient        = VERIFY_UNSET; /* Will become VERIFY_NONE */
+    rec->bFakeBasicAuth       = UNSET;        /* Will become FALSE */
+    rec->tSessionCacheTimeout = 0;
+
+    rec->fileLogFile          = NULL;
     rec->px509Certificate     = NULL;
     rec->prsaKey              = NULL;
     rec->pSSLCtx              = NULL;
-    rec->bFakeBasicAuth       = UNSET;        /* Will become FALSE */
-    rec->tSessionCacheTimeout = 0;
 
     return rec;
 }
@@ -284,6 +285,11 @@ static void *config_server_merge(pool *p, void *basev, void *addv)
     cfgMergeBool(bFakeBasicAuth);
     cfgMergeInt(tSessionCacheTimeout);
 
+    cfgMerge(fileLogFile, NULL);
+    cfgMerge(px509Certificate, NULL);
+    cfgMerge(prsaKey, NULL);
+    cfgMerge(pSSLCtx, NULL);
+
     return new;
 }
 
@@ -485,18 +491,10 @@ static void init_Module(server_rec *s, pool *p)
     char *szLogFile;
     char buf[MAX_STRING_LEN];
     static int nStashStdin, nStashStderr, nSaveStdin, nSaveStderr;
+    piped_log *pl;
     char *cp;
     char *cp2;
 
-#ifdef SHARED_MODULE
-    /* 
-     *  Allegedly this is only run once when included as a DSO (though the
-     *  value of doing that is questionable, since the rest of the code
-     *  doesn't work without this module).
-     */
-    g_bFirstTime = FALSE;
-#endif
-
     /*  
      *  skip nasty first time initialisation of Apache API
      *  to avoid problems with SSL initialization
@@ -583,7 +581,7 @@ static void init_Module(server_rec *s, pool *p)
          */
         if (!pConfig->tSessionCacheTimeout) {
             ap_log_error(APLOG_MARK, APLOG_NOERRNO|APLOG_ERR, s,
-                         "mod_ssl: No SSL Session Cache Timeout set [hint: SSLCacheTimeOut]");
+                         "mod_ssl: No SSL Session Cache Timeout set [hint: SSLSessionCacheTimeout]");
             ap_clear_pool(p);
             exit(1);
         }
@@ -594,14 +592,6 @@ static void init_Module(server_rec *s, pool *p)
             ap_clear_pool(p);
             exit(1);
         }
-        if (pConfig->nVerifyClient < 0 || 
-            pConfig->nVerifyClient > VERIFY_OPTIONAL_NO_CA) {
-            ap_log_error(APLOG_MARK, APLOG_NOERRNO|APLOG_ERR, s,
-                         "mod_ssl: Bad value for SSLVerifyClient (%d)", 
-                         pConfig->nVerifyClient);
-            ap_clear_pool(p);
-            exit(1);
-        }
         if (!pConfig->szLogFile) {
             ap_log_error(APLOG_MARK, APLOG_NOERRNO|APLOG_ERR, s,
                          "mod_ssl: No SSL logfile set [hint: SSLLogFile]");
@@ -612,16 +602,33 @@ static void init_Module(server_rec *s, pool *p)
         /*
          * Open the SSL logfile
          */
-        szLogFile = ap_server_root_relative(p, pConfig->szLogFile);
-        pConfig->fileLogFile = ap_pfopen(p, szLogFile, "a");
-        if (!pConfig->fileLogFile) {
-            ap_log_error(APLOG_MARK, APLOG_ERR, s,
-                         "mod_ssl: Cannot open SSL logfile %s",
-                         pConfig->szLogFile);
-            ap_clear_pool(p);
-            exit(1);
+        if (pConfig->szLogFile != NULL) {
+            if (strNE(pConfig->szLogFile, "/dev/null")) {
+                if (pConfig->szLogFile[0] == '|') {
+                    szLogFile = ap_server_root_relative(p, pConfig->szLogFile+1);
+                    if ((pl = ap_open_piped_log(p, szLogFile)) == NULL) {
+                        ap_log_error(APLOG_MARK, APLOG_ERR, s,
+                                     "mod_ssl: Cannot open reliable pipe to SSL logfile filter %s",
+                                     szLogFile);
+                        ap_clear_pool(p);
+                        exit(1);
+                    }
+                    pConfig->fileLogFile = ap_pfdopen(p, ap_piped_log_write_fd(pl), "a");
+                    setbuf(pConfig->fileLogFile, NULL);
+                }
+                else {
+                    szLogFile = ap_server_root_relative(p, pConfig->szLogFile);
+                    if ((pConfig->fileLogFile = ap_pfopen(p, szLogFile, "a")) == NULL) {
+                        ap_log_error(APLOG_MARK, APLOG_ERR, s,
+                                     "mod_ssl: Cannot open SSL logfile %s",
+                                     szLogFile);
+                        ap_clear_pool(p);
+                        exit(1);
+                    }
+                    setbuf(pConfig->fileLogFile, NULL);
+                }
+            }
         }
-        setbuf(pConfig->fileLogFile, NULL);
 
         /*
          * Read the server certificate and key
@@ -669,14 +676,6 @@ static void init_SSLeay(server_rec *s, pool *p)
     __SSLeay ERR_load_crypto_strings();
     __SSLeay SSLeay_add_ssl_algorithms();
 
-    /* 
-     *  For debugging purposes one can carefully enable this.
-     *  But remember that this can be used for /tmp symlink attacks.
-     */
-#if DEBUG_SSLEAY
-    __SSLeay SSL_debug("/tmp/ssldebug");
-#endif
-
     return;
 }
 
